#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <dune/fufem/hdf5/singletonwriter.hh>

#include "frictionalboundary-writer.hh"
#include "restrict.hh"

template <class GridView>
template <class Vector>
FrictionalBoundaryWriter<GridView>::FrictionalBoundaryWriter(
    HDF5::Group& group, const Vector& vertexCoordinates,
    const Patch& frictionalBoundary)
    : group_(group),
      frictionalBoundary_(frictionalBoundary),
      frictionalBoundaryDisplacementWriter_(group_, "displacement",
                                            frictionalBoundary.numVertices(),
                                            Vector::block_type::dimension),
      frictionalBoundaryVelocityWriter_(group_, "velocity",
                                        frictionalBoundary.numVertices(),
                                        Vector::block_type::dimension),
      frictionalBoundaryStateWriter_(group_, "state",
                                     frictionalBoundary.numVertices()),
      frictionalBoundaryCoefficientWriter_(group_, "coefficient",
                                           frictionalBoundary.numVertices()) {
  auto const frictionalBoundaryCoordinates =
      restrictToSurface(vertexCoordinates, frictionalBoundary);
  HDF5::SingletonWriter<2> frictionalBoundaryCoordinateWriter(
      group_, "coordinates", frictionalBoundaryCoordinates.size(),
      Vector::block_type::dimension);
  setEntry(frictionalBoundaryCoordinateWriter, frictionalBoundaryCoordinates);
}

template <class GridView>
template <class Vector, class ScalarVector>
void FrictionalBoundaryWriter<GridView>::write(
    const size_t timeStep, const Vector& u, const Vector& v, const ScalarVector& alpha, const ScalarVector& frictionCoeff) {

  auto const frictionalBoundaryDisplacements = restrictToSurface(u, frictionalBoundary_);
  addEntry(frictionalBoundaryDisplacementWriter_, timeStep, frictionalBoundaryDisplacements);

  auto const frictionalBoundaryVelocities = restrictToSurface(v, frictionalBoundary_);
  addEntry(frictionalBoundaryVelocityWriter_, timeStep, frictionalBoundaryVelocities);

  auto const frictionalBoundaryStates = restrictToSurface(alpha, frictionalBoundary_);
  addEntry(frictionalBoundaryStateWriter_, timeStep, frictionalBoundaryStates);

  auto const frictionalBoundaryCoefficient = restrictToSurface(frictionCoeff, frictionalBoundary_);
  addEntry(frictionalBoundaryCoefficientWriter_, timeStep, frictionalBoundaryCoefficient);
}

#include "frictionalboundary-writer_tmpl.cc"
